﻿using BeSafePlus;
using BeSafePlus.ViewModels;
using BeSafePlus.Views;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using CommunityToolkit.Mvvm.Messaging;
using Microsoft.Maui.Controls;

namespace BeSafePlus
{
    public partial class AppShell : Shell
    {
        private readonly SettingsViewModel _settingsViewModel;

        private ShellSection _activityHistorySection;
        public AppShell(SettingsViewModel svm)
        {
            InitializeComponent();

            _settingsViewModel = svm;

            BindingContext = _settingsViewModel;

            // registrer rute som tillader shell at pushe nye sider
            Routing.RegisterRoute(nameof(QrScannerPage), typeof(QrScannerPage));

            _activityHistorySection = new ShellSection
            {
                Title = "Aktivitetshistorik",
                Icon = "activityhistory.png",
                Route = "ActivityHistory"
            };

            _activityHistorySection.Items.Add(new ShellContent
            {
                Title = "Aktivitetshistorik",
                ContentTemplate = new DataTemplate(typeof(ActivityHistoryPage))
            });

            WeakReferenceMessenger.Default.Register<ActivityHistoryChangedMessage>(this, (r, msg) =>
            {
                UpdateActivityHistoryTab(msg.IsEnabled);
            });

            UpdateActivityHistoryTab(_settingsViewModel.IsActivityHistoryEnabled);
        }
        private void UpdateActivityHistoryTab(bool isEnabled)
        {
            if (isEnabled)
            {
                if (!Items.Contains(_activityHistorySection))
                {
                    Items.Add(_activityHistorySection);
                }
            }

            else
            {
                if (Items.Contains(_activityHistorySection))
                {
                    Items.Remove(_activityHistorySection);
                }
            }
        }
    }
}