﻿using BeSafePlus.Database;
using BeSafePlus.Models.Domain;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;

namespace BeSafePlus.Clients
{
    public class RedCapClient : IRedCapClient
    {
        private readonly HttpClient _httpClient;

        private readonly ILocalDatabase _localDatabase;
        private JsonSerializerOptions SerializerOptions => new()
        {
            PropertyNamingPolicy = JsonNamingPolicy.CamelCase,
            WriteIndented = true,
        };
        public RedCapClient(ILocalDatabase localDatabase) 
        {
            _httpClient = new HttpClient();

            _localDatabase = localDatabase;
        }
        public async Task PostReport(long userId, DateTime reportDay, ActivityReport activityReport, Steps steps, List<string>? errors = null)
        {
            if (activityReport == null || steps == null)
            {
                return;
            }

            // get endpoint and token from localdatabase?
            var endpoint = "https://whec11ea74715f627381.free.beeceptor.com";

            var redCapEntry = new RedCapEntry
            {
                UserId = userId,
                ActivityReport = activityReport,
                Steps = steps,
                Errors = errors ?? [],
                ReportDate = reportDay,
            };

            try
            {
                var report = JsonSerializer.Serialize<RedCapEntry>(redCapEntry, SerializerOptions);

                var content = new StringContent(report, Encoding.UTF8, "application/json");

                TestingTools.TestingTools.LogData(report, "beSafeActivityReport", "json", true);

                var request = new HttpRequestMessage(HttpMethod.Post, endpoint)
                {
                    Content = content
                };

                var response = await _httpClient.SendAsync(request);

                if (response.IsSuccessStatusCode)
                {
                    Debug.WriteLine("All good, report was sent");
                }

                else
                {
                    Debug.WriteLine($"{response.StatusCode}");
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine($"Error in {nameof(RedCapClient)}: {ex.Message}");
            }
        }
    }
}