﻿using BeSafePlus.Sensors;
using SQLite;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BeSafePlus.Models
{
    public class ActivityLevel
    {
        [PrimaryKey, AutoIncrement]
        public int Id { get; set; }
        public string UserActivityLevel { get; set; } = string.Empty;
        public string Time { get; set; } = string.Empty; // Should be used to calculate activity level percentages
    }
    public class ActivityLevelDto
    {
        public ActivityLevelDto() { }
        public SensorEventHandler.UserActivityState ActivityState { get; set; }
        public DateTime Time { get; set; }
        public static List<ActivityLevelDto> ConvertToDtoList(IEnumerable<ActivityLevel> activityLevels)
        {
            var result = activityLevels
                .Select(al => new ActivityLevelDto
                {
                    ActivityState = Enum.TryParse(al.UserActivityLevel, out SensorEventHandler.UserActivityState parsedState)
                        ? parsedState
                        : throw new InvalidOperationException($"Invalid UserActivityLevel: {al.UserActivityLevel}"),

                    Time = DateTime.TryParseExact(al.Time, "yyyy-MM-dd HH:mm", null, System.Globalization.DateTimeStyles.None, out DateTime parsedTime)
                        ? parsedTime
                        : throw new InvalidOperationException($"Invalid Time format: {al.Time}")
                })
                .ToList();

            return result;
        }
    }
}