using Android.App;
using Android.Content;
using Android.Content.PM;
using Android.Hardware;
using Android.Runtime;
using Android.Util;
using BeSafePlus.Sensors;
using System.Diagnostics;
using AndroidX.Core.App;

namespace BeSafePlus.Platforms.Android.AndroidSensor
{
    public class ActivityTrackingSensor : Java.Lang.Object, ISensorEventListener, IActivityTrackingSensor
    {
        private SensorManager? _sensorManager;

        private Sensor? _stepSensor;

        private Sensor? _accelerometerSensor;

        private int _currentBaseStepCount;

        private bool _isFirstReading = true; // true on first load

        public event Func<Task>? ResetStepCountEvent;

        public event Action<int, DateTime>? StepCounterChangeEvent;

        public event Action<float, float, float, DateTime>? AccelerometerChangeEvent;
        public ActivityTrackingSensor() { }
        public void Start()
        {
            _sensorManager = global::Android.App.Application.Context.GetSystemService(Context.SensorService) as SensorManager;

            if (_sensorManager == null)
            {
                Log.Error("ActivityTrackingSensor", "Sensormanager null");

                throw new InvalidOperationException("sensor manager is null");
            }

            _stepSensor = _sensorManager?.GetDefaultSensor(SensorType.StepCounter) ?? throw new NotSupportedException($"Could not initialize step counter sensor");

            _accelerometerSensor = _sensorManager?.GetDefaultSensor(SensorType.Accelerometer) ?? throw new NotSupportedException($"Could not initialize accelerometer sensor");

            if (_stepSensor != null || _accelerometerSensor != null)
            {
                _sensorManager?.RegisterListener(this, _stepSensor, SensorDelay.Ui);

                _sensorManager?.RegisterListener(this, _accelerometerSensor, SensorDelay.Normal);

                Log.Info("ActivityTrackingSensor", "Sensors registered sucessfully");
                
                return;
            }

            var msg = $"Sensor {(_stepSensor?.Type.ToString() ?? _accelerometerSensor?.Type.ToString()) ?? "NONE"} was not available";

            Log.Error("ActivityTrackingSensor", msg);
        }
        public void Stop()
        {
            _sensorManager?.UnregisterListener(this);
        }
        public void OnAccuracyChanged(Sensor? sensor, [GeneratedEnum] SensorStatus accuracy)
        {
            Debug.WriteLine("accarcuychanged");
        }
        public void OnSensorChanged(SensorEvent? e)
        {
            var sensor = e?.Sensor;

            var values = e?.Values;

            if (sensor == null || values == null || values.Count == 0)
            {
                Log.Info("ActivityTrackingSensor", "Event from unknown sensor source");

                return;
            }

            switch (sensor.Type)
            {
                case SensorType.Accelerometer:
                    HandleAccelerometerEvent(values);
                    break;

                case SensorType.StepCounter:
                    HandleStepCounterEvent(values);                    
                    break;
            }
        }
        private void HandleStepCounterEvent(IList<float> values)
        {
            // Reads accumulated steps since last system reboot
            var stepsRead = (int)values[0];

            if (_isFirstReading)
            {
                _isFirstReading = false;

                _currentBaseStepCount = stepsRead;
            }

            // Get new steps since last events
            var newSteps = stepsRead - _currentBaseStepCount;

            // Update basesteps to substract correctly
            _currentBaseStepCount = stepsRead;

            StepCounterChangeEvent?.Invoke(newSteps, DateTime.Now);
        }
        private void HandleAccelerometerEvent(IList<float> values)
        {
            if (values.Count >= 3)
            {
                AccelerometerChangeEvent?.Invoke(values[0], values[1], values[2], DateTime.Now);
            }
        }
        public async Task ResetDailySteps()
        {
            if (ResetStepCountEvent != null)
            {
                await ResetStepCountEvent.Invoke();
            }
        }
    }
}