using Android.App;
using Android.Content;
using Android.Content.PM;
using Android.OS;
using Android.Runtime;
using Android.Util;
using AndroidX.Core.App;
using BeSafePlus.Extensions;
using BeSafePlus.Platforms.Android.CustomBroadcastReceivers;
using BeSafePlus.Platforms.Android.Scheduling;
using BeSafePlus.Sensors;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BeSafePlus.Platforms.Android.AndroidSensor
{
    // https://stackoverflow.com/questions/71259615/how-to-create-a-background-service-in-net-maui
    // https://developer.android.com/develop/background-work/services/fgs/service-types#health 
    // HealthType is best for "exercise trackers". requires certain permissions FOREGROUND_SERVICE_HEALTH in manifest and atleast ACTIVITY_RECOGNITION. 
    // TODO: check API support for this type
    [Service(Enabled = true, Exported = false, ForegroundServiceType = ForegroundService.TypeHealth)]
    public class ActivityTrackingService : Service
    {
        private IActivityTrackingSensor? _activityTrackingSensor;
        private int startCounter = 0;

        private IScheduleService? _scheduleService;

        private IServiceProvider? _serviceProvider;

        public const int NOTIFIY_ID = 431831;

        public const string CHANNEL_ID = "ActivityTrackingSensorChannel";

        // Indicates whether this service is running or not
        public static bool IsRunning { get; private set; } = false;

        // android uses reflection = parameterless constructor, use their microsoft maui api to get service provider
        public ActivityTrackingService()
        {
           
        }

        // Use lifecycle hooks to handle service
        public override void OnCreate()
        {
            base.OnCreate();

            Log.Info("bongotrummer", "bblsonder");

            // StartForeground(NOTIFIY_ID, CreateNotification(), ForegroundService.TypeHealth);

           
        }
        public override void OnDestroy()
        {
            _activityTrackingSensor?.Stop();

            IsRunning = false;

            TestingTools.TestingTools.LogData($"service stoppet {DateTime.Now}", "servicefejl");
            
            base.OnDestroy();

           // FirebaseHelper.UpdateServiceState(false);
        }
        public override IBinder? OnBind(Intent? intent) => null;

        [return: GeneratedEnum]
        public override StartCommandResult OnStartCommand(Intent? intent, [GeneratedEnum] StartCommandFlags flags, int startId)
        {
            StartForeground(NOTIFIY_ID, CreateNotification(), ForegroundService.TypeHealth);

            startCounter++;
            TestingTools.TestingTools.LogData($"service startet ({startCounter}): {DateTime.Now}", "servicefejl");

            var serviceProvider = IPlatformApplication.Current?.Services;

            _activityTrackingSensor = serviceProvider?.GetService<IActivityTrackingSensor>();

            _scheduleService = serviceProvider?.GetRequiredService<IScheduleService>();

            if (_activityTrackingSensor == null)
            {
                Log.Error("ActivityTrackingService", "IActivityTrackingSensor not resolved.");

                StopSelf();  // Stops the service to prevent further crashes


            }

            else
            {
                Log.Info("ActivityTrackingService", "IActivityTrackingSensor resolved successfully.");
            }

            try
            {
                _activityTrackingSensor?.Start();

                _scheduleService?.ScheduleAlarm<MidnightResetReceiver>(AndroidRequestCodes.MidnightReset, TimeExtensions.GetTimeUntilMidnight() /*TimeSpan.FromSeconds(5)*/);
                //_scheduleService?.ScheduleAlarm<HealthCheckReceiver>(AndroidRequestCodes.HealthCheck, TimeSpan.FromSeconds(5));
                _scheduleService?.ScheduleAlarm<MidnightReportReceiver>(AndroidRequestCodes.MidnightReport, TimeExtensions.GetTimeUntilMidnight()+TimeSpan.FromMinutes(10) /*TimeSpan.FromSeconds(5)*/);

                IsRunning = true;
            }

            catch (Exception ex)
            {
                Log.Error("ActivityTrackingService", $"error starting sensor{ex.Message}");
            }

            return StartCommandResult.Sticky;
        }
        private Notification CreateNotification()
        {
            var manager = GetSystemService(NotificationService) as NotificationManager;

            if (OperatingSystem.IsAndroidVersionAtLeast(26))
            {
                var channel = new NotificationChannel(CHANNEL_ID, "BeSAFEAcitivityService", NotificationImportance.Max);

                manager?.CreateNotificationChannel(channel);
            }

            /* make clickable to open mainacitivity */
            var intent = new Intent(this, typeof(MainActivity));
            intent.SetFlags(ActivityFlags.ClearTop | ActivityFlags.SingleTop);
            var openAppIntent = PendingIntent.GetActivity(this, 0, intent, PendingIntentFlags.Immutable);

            // https://developer.android.com/reference/kotlin/androidx/core/app/NotificationCompat.Builder
            return new NotificationCompat.Builder(this, CHANNEL_ID)
                 .SetContentTitle("BESafe krer i baggrunden")
                 .SetContentText("Appen mler skridt og aktivitet")
                 .SetSmallIcon(Microsoft.Maui.Resource.Drawable.abc_ab_share_pack_mtrl_alpha)
                 .SetOngoing(true)
                 .SetContentIntent(openAppIntent)
                 .Build();
        }
    }
}