﻿using Android.App;
using Android.Content;
using Android.Util;
using BeSafePlus.Platforms.Android.AndroidSensor;
using BeSafePlus.Platforms.Android.Scheduling;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BeSafePlus.Platforms.Android.CustomBroadcastReceivers
{
    // https://developer.android.com/develop/background-work/background-tasks/broadcasts
    // https://stackoverflow.com/questions/49403069/xamarin-android-receiver-on-boot-completed-error
    [BroadcastReceiver(Enabled = true, Exported = true, Permission = "android.permission.RECEIVE_BOOT_COMPLETED")]
    [IntentFilter([Intent.ActionBootCompleted])]
    public class BootReceiver : BroadcastReceiver
    {
        public override void OnReceive(Context? context, Intent? intent)
        {
            if (intent?.Action == Intent.ActionBootCompleted && context != null)
            {
                var serviceIntent = new Intent(context, typeof(ActivityTrackingService));

                if (OperatingSystem.IsAndroidVersionAtLeast(26))
                {
                    //SW: I added the below - to ensure the service does not cause the 
                    //app to crash ... however - you could consider using a JobIntentService
                    //and schedule the service to run at a later time ....
                    try
                    {
                        var scheduleService = IPlatformApplication.Current?.Services?.GetService<IScheduleService>();
                        scheduleService?.ScheduleAlarm<HealthCheckReceiver>(AndroidRequestCodes.HealthCheck, TimeSpan.FromMinutes(5));
                        // context.StartForegroundService(serviceIntent);
                    }

                    catch (Exception ex)
                    {
                        Log.Error("BeSafePlus", ex.Message); //SW: I added this and check to see what happens.
                    }
                    //context.StartForegroundService(serviceIntent); SW: I removed this
                }
            }
        }
    }
}