﻿using Android.App;
using Android.Content;
using Android.Util;
using BeSafePlus.Services; // Import your Firestore service
using BeSafePlus.Platforms.Android.Scheduling;
using System;
using System.Linq;
using System.Threading.Tasks;
using BeSafePlus.Platforms.Android.AndroidSensor;

namespace BeSafePlus.Platforms.Android.CustomBroadcastReceivers
{
    [BroadcastReceiver(Enabled = true, Exported = false)]
    public class HealthCheckReceiver : BroadcastReceiver
    {
        bool serviceRunning = false;
        public override async void OnReceive(Context? context, Intent? intent)
        {
            try
            {
                var manager = context?.GetSystemService(Context.ActivityService) as ActivityManager;
                var runningServices = manager?.GetRunningServices(int.MaxValue);

                // Check if ActivityTrackingService is running
                var isTrackingServiceRunning = runningServices?.Any(s => s.Service?.ClassName == typeof(ActivityTrackingService).Name);


                if (isTrackingServiceRunning != null)
                {
                    serviceRunning = (bool)isTrackingServiceRunning;
                    if (!serviceRunning)
                    {
                        Log.Warn("ServiceHealthCheck", "ActivityTrackingService was not running. Restarting...");
                        if (context != null)
                        {
                            var serviceIntent = new Intent(context, typeof(ActivityTrackingService));
                            context.StartForegroundService(serviceIntent);
                            Log.Warn("ServiceHealthCheck", "ActivityTrackingService started sucessfully");

                        }
                    }
                    else
                    {
                        Log.Debug("ServiceHealthCheck", "Activity service is still running.");
                    }
                }
                else
                {
                    Log.Debug("ServiceHealthCheck", "GetRunningServices returned null.");
                }
            }
            catch (Exception ex) // Escalation 1
            {
                Log.Error("ServiceHealthCheck", $"Error occurred during healthcheck: {ex.Message}");
                Log.Error("ServiceHealthCheck", $"Attempting to wake app with push notifications....");

                try
                {
                    var firestoreService = IPlatformApplication.Current?.Services?.GetService<FirestoreService>();

                    if (firestoreService != null)
                    {
                        var deviceId = "010101";
                        await firestoreService.InsertServiceStatus(deviceId, serviceRunning);
                    }
                }
                catch(Exception ex2) // Escalation 2
                {
                    Log.Error("ServiceHealthCheck", $"Error occurred during healthcheck, when trying to wake app with push notifications: {ex.Message}");
                    Log.Error("ServiceHealthCheck", $"Sending post notification to use as last resort");
                    // Send post notification

                    // if it fails catch from outer or alert redcap

                }
            }
            var scheduleService = IPlatformApplication.Current?.Services?.GetService<IScheduleService>();
            scheduleService?.ScheduleAlarm<HealthCheckReceiver>(AndroidRequestCodes.HealthCheck, TimeSpan.FromHours(2));
        }
    }
}
