﻿using Android.Content;
using Android.Util;
using BeSafePlus.Extensions;
using BeSafePlus.Platforms.Android.Scheduling;
using BeSafePlus.Sensors;
namespace BeSafePlus.Platforms.Android.CustomBroadcastReceivers
{
    [BroadcastReceiver(Enabled = true, Exported = false)]
    public class MidnightResetReceiver : BroadcastReceiver
    {
        public override void OnReceive(Context? context, Intent? intent)
        {
            var pendingResult = GoAsync();

            Task.Run(async () =>
            {
                try
                {
                    await ResetDailySteps();
                }
                catch (Exception ex)
                {
                    Log.Error("MidnightResetReceiver", $"Exception during ResetDailySteps: {ex.Message}");
                }
                finally
                {
                    pendingResult?.Finish();
                }
            });
        }
        private static async Task ResetDailySteps()
        {
            var serviceProvider = IPlatformApplication.Current?.Services;

            var activityTrackingSensor = serviceProvider?.GetService<IActivityTrackingSensor>();

            var scheduleService = serviceProvider?.GetService<IScheduleService>();

            if (activityTrackingSensor != null && scheduleService != null)
            {
                await activityTrackingSensor.ResetDailySteps();

                scheduleService.ScheduleAlarm<MidnightResetReceiver>(AndroidRequestCodes.MidnightReset, TimeExtensions.GetTimeUntilMidnight());

                return;
            }
            Log.Error("MidnightResetReceiver", "Step counter could not be reset at midnight");
        }
    }
}