﻿using Android.App;
using Android.Content;
using Android.Content.OM;
using Android.Content.PM;
using Android.OS;
using Android.Widget;
using BeSafePlus.Database;
using BeSafePlus.Platforms.Android.AndroidSensor;
using System.Runtime.Versioning;

namespace BeSafePlus
{
    // This call site is reachable on: 'Android X.X' and later. '*' is only supported on: 'Android X.X' and later.
    [Activity(Theme = "@style/Maui.SplashTheme", MainLauncher = true, LaunchMode = LaunchMode.SingleTop, ConfigurationChanges = ConfigChanges.ScreenSize | ConfigChanges.Orientation | ConfigChanges.UiMode | ConfigChanges.ScreenLayout | ConfigChanges.SmallestScreenSize | ConfigChanges.Density)]
    public class MainActivity : MauiAppCompatActivity
    {
        private readonly ILocalDatabase? _localDatabase;

        private static readonly string[] RequiredPermissions = {
            Android.Manifest.Permission.ActivityRecognition,
            Android.Manifest.Permission.PostNotifications
        };
        
        private const int ActivityPermissionRequestCode = 1234;
        public MainActivity() 
        {
            _localDatabase = IPlatformApplication.Current?.Services?.GetService<ILocalDatabase>();
        }
        protected override async void OnCreate(Bundle? savedInstanceState)
        {
            base.OnCreate(savedInstanceState);

            if (_localDatabase != null)
            {
                var user = await _localDatabase.GetUser();

                if (user == null)
                {
                    await _localDatabase.SetUserId();
                }
            }

            var permissions = CheckPermissions();

            if (permissions.Length != 0)
            {
                // Request permissions and start service on OnRequestPermissionsResult == granted
                RequestPermissions(permissions);
            }

            else
            {
                // Permissions already granted
                StartActivityTrackingService();
            }
        }
        public override void OnRequestPermissionsResult(int requestCode, string[] permissions, Permission[] grantResults)
        {
            base.OnRequestPermissionsResult(requestCode, permissions, grantResults);

            if (requestCode == ActivityPermissionRequestCode)
            {
                if (grantResults.All(r => r == Permission.Granted))
                {
                    StartActivityTrackingService();
                }
            }
        }
        private string[] CheckPermissions()
        {
            return RequiredPermissions.Where(permission => CheckSelfPermission(permission) != Permission.Granted).ToArray();
        }
        private void RequestPermissions(string[] permissions)
        {
            if (OperatingSystem.IsAndroidVersionAtLeast(29))
            {
                RequestPermissions(permissions, ActivityPermissionRequestCode);
            }
        }
        private void StartActivityTrackingService()
        {
            if (ActivityTrackingService.IsRunning is false)
            {
                var intent = new Intent(this, typeof(ActivityTrackingService));

                if (OperatingSystem.IsAndroidVersionAtLeast(26))
                {
                    StartForegroundService(intent);
                }

                else
                {
                    StartService(intent);
                }
            }
        }
    }
}