﻿using Android.App;
using Android.Content;
using Android.OS;
using Android.Util;

namespace BeSafePlus.Platforms.Android.Scheduling
{
    public class BeSafeScheduleService : IScheduleService
    {
        public void ScheduleAlarm<TReceiver>(int requestCode, TimeSpan timeSpan)
            where TReceiver : BroadcastReceiver
        {
            try
            {
                var context = global::Android.App.Application.Context;

                if (context.GetSystemService(Context.AlarmService) is not AlarmManager alarmManager)
                {
                    return;
                }

                var intent = new Intent(context, typeof(TReceiver));

                // do not schedule a new alarm if one of same type already exists. could be because fgs was retstarted by system
                var existingIntent = PendingIntent.GetBroadcast(context, requestCode, intent, PendingIntentFlags.NoCreate | PendingIntentFlags.Immutable);
                if (existingIntent != null)
                {
                    Log.Info("BeSafeScheduleService", $"Alarm for {typeof(TReceiver).Name} with request code {requestCode} already scheduled.");
                    return;
                }

                var flags = PendingIntentFlags.UpdateCurrent;

                if (Build.VERSION.SdkInt >= BuildVersionCodes.S) // API 31+
                {
                    flags |= PendingIntentFlags.Immutable;
                }

                var pendingIntent = PendingIntent.GetBroadcast(context, requestCode, intent, flags);

                var midnightMillis = DateTimeOffset.Now.ToUnixTimeMilliseconds() + (long)timeSpan.TotalMilliseconds;

                if (pendingIntent != null)
                {
                    // If exact timing permissions are allowed
                    if (alarmManager.CanScheduleExactAlarms())
                    {
                        alarmManager.SetExactAndAllowWhileIdle(AlarmType.RtcWakeup, midnightMillis, pendingIntent);
                    }

                    else
                    {
                        // Use inexact
                        alarmManager.SetAndAllowWhileIdle(AlarmType.RtcWakeup, midnightMillis, pendingIntent);
                        // or invoke event for user to allow it
                    }
                    Log.Info("BeSafeScheduleService", $"Alarm was scheduled with receiver: {typeof(TReceiver).Name}");
                }
            }
            catch (Exception e)
            {
                Log.Error("BeSafeScheduleService", $"Failed to schedule alarm for {typeof(TReceiver)}, with error: {e.Message}");

                throw;
            }
        }
    }
}