﻿using Android.Content;
using Android.Util;
using AndroidX.Work;
using BeSafePlus.Extensions;
using BeSafePlus.Platforms.Android.CustomBroadcastReceivers;
using BeSafePlus.Platforms.Android.Scheduling;
using BeSafePlus.Services.Interfaces;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BeSafePlus.Platforms.Android.Workers
{
    public class BeSafeReportWorker : Worker
    {
        public BeSafeReportWorker(Context context, WorkerParameters workerParams) : base(context, workerParams) { }
        public override Result DoWork()
        {
            if(RunAttemptCount >= 5)
            {
                Log.Error("BeSafeReportWorker", "Report generation failed after 5 attemps!!!");

                return Result.InvokeFailure();
            }

            var stopwatch = new Stopwatch();

            stopwatch.Start();

            Log.Debug("BeSafeReportWorker", "Starting report writing");

            var serviceProvider = IPlatformApplication.Current?.Services;

            var reportService = serviceProvider?.GetService<IReportService>();

            var scheduleService = serviceProvider?.GetService<IScheduleService>();

            try
            {
                if (reportService != null && scheduleService != null)
                {
                    reportService.WriteAndSendReport().GetAwaiter().GetResult();

                    stopwatch.Stop();

                    Log.Info("BeSafeReportWorker", $"Report was written and send to database. Elapsed: {stopwatch.ElapsedMilliseconds}. Attempt: {RunAttemptCount}");

                    scheduleService.ScheduleAlarm<MidnightReportReceiver>(AndroidRequestCodes.MidnightReport, TimeExtensions.GetTimeUntilMidnight()+ TimeSpan.FromMinutes(5));

                    return Result.InvokeSuccess();
                }
            }

            catch(Exception ex)
            {
                Log.Error("BeSafeReportWorker", $"Error when trying to generate report: {ex.Message}");
            }

            stopwatch.Stop();

            Log.Info("BeSafeReportWorker", $"Report generation failed. Elapsed: {stopwatch.ElapsedMilliseconds}. Attempting to try again... (Attempt:{RunAttemptCount})");

            return Result.InvokeRetry();
        }
    }
}