﻿using Google.Cloud.Firestore;
using System;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;
using Microsoft.Maui.Storage;

namespace BeSafePlus.Services
{
    public class FirestoreService
    {
        private FirestoreDb db;

        // Setup Firestore connection using Admin SDK credentials
        private async Task SetupFirestore()
        {
            if (db == null)
            {
                var stream = await FileSystem.OpenAppPackageFileAsync("admin-sdk.json");
                var reader = new StreamReader(stream);
                var contents = reader.ReadToEnd();

                db = new FirestoreDbBuilder
                {
                    ProjectId = "besafeplus-1d01d",
                    JsonCredentials = contents  // Set the credentials from the admin-sdk.json file
                }.Build();
            }
        }

        // Insert service status into Firestore
        public async Task InsertServiceStatus(string deviceId, bool serviceRunning)
        {
            await SetupFirestore();
            var status = new Dictionary<string, object>
            {
                { "serviceRunning", serviceRunning },
                { "timestamp", Timestamp.FromDateTime(DateTime.UtcNow) } // Explicitly convert DateTime to Firestore Timestamp
            };

            // Insert status in "ServiceStatus" collection
            await db.Collection("ServiceStatus").Document(deviceId).SetAsync(status);
        }

        // Get the status of the service from Firestore
        public async Task<bool> GetServiceStatus(string deviceId)
        {
            await SetupFirestore();
            var documentSnapshot = await db.Collection("ServiceStatus").Document(deviceId).GetSnapshotAsync();
            if (documentSnapshot.Exists)
            {
                var data = documentSnapshot.ToDictionary();
                return data.ContainsKey("serviceRunning") && (bool)data["serviceRunning"];
            }
            return false; // Default if the document doesn't exist
        }

        // Insert the FCM Token into Firestore
        public async Task InsertFCMToken(string deviceId, string token)
        {
            await SetupFirestore();
            var tokenData = new Dictionary<string, object>
            {
                { "fcmToken", token },
                { "timestamp", Timestamp.FromDateTime(DateTime.UtcNow) } // Explicitly convert DateTime to Firestore Timestamp
            };

            // Insert the FCM token into the "FCMTokens" collection
            await db.Collection("FCMTokens").Document(deviceId).SetAsync(tokenData);
        }

        // Get the FCM Token for a device from Firestore
        public async Task<string> GetFCMToken(string deviceId)
        {
            await SetupFirestore();
            var documentSnapshot = await db.Collection("FCMTokens").Document(deviceId).GetSnapshotAsync();
            if (documentSnapshot.Exists)
            {
                var data = documentSnapshot.ToDictionary();
                return data.ContainsKey("fcmToken") ? (string)data["fcmToken"] : null;
            }
            return null; // Return null if no token is found
        }
    }
}
