using BeSafePlus.Database;
using BeSafePlus.ViewModels;
using Microsoft.Maui.Controls;
using System.Reflection.Emit;
using System.Text.RegularExpressions;

namespace BeSafePlus
{
    public partial class SettingsPage : ContentPage
    {
        ILocalDatabase _localDatabase;
        public SettingsPage(SettingsViewModel vm, ILocalDatabase localDatabase)
        {
            InitializeComponent();

            BindingContext = vm;
            
            _localDatabase = localDatabase;

            SetUserId();
        }
        internal async void SetUserId()
        {
            var userId = await _localDatabase.GetUser();

            if (userId != null)
            {
                UserIdLabel.Text = string.Join(" ", userId.Id.ToString().ToCharArray());

                userBarcode.Value = $"{userId.Id}";
            }
        }
        private void OnNameEntryTextChanged(object sender, TextChangedEventArgs e)
        {
            Entry entry = (Entry)sender;

            string filteredText = new string(e.NewTextValue.Where(c => char.IsLetter(c) || c == ' ').ToArray());

            filteredText = Regex.Replace(filteredText, @"\s+", " ");

            if (filteredText.Length > 25)
            {
                filteredText = filteredText.Substring(0, 25);
            }

            if (entry.Text != filteredText)
            {
                entry.Text = filteredText;
            }
        }
        private void OnAgeEntryTextChanged(object sender, TextChangedEventArgs e)
        {
            Entry entry = (Entry)sender;

            string filteredText = e.NewTextValue;

            filteredText = new string(filteredText.Where(c => char.IsDigit(c)).ToArray());

            if (filteredText.Length > 3)
            {
                filteredText = filteredText.Substring(0, 3);
            }

            if (filteredText.StartsWith("0") && filteredText.Length > 1)
            {
                filteredText = filteredText.Substring(1);
            }

            if (filteredText == "0")
            {
                filteredText = "";
            }

            if (entry.Text != filteredText)
            {
                entry.Text = filteredText;
            }
        }
        private void OnHeightEntryTextChanged(object sender, TextChangedEventArgs e)
        {
            Entry entry = (Entry)sender;

            string filteredText = e.NewTextValue;

            filteredText = new string(filteredText.Where(c => char.IsDigit(c)).ToArray());

            if (filteredText.Length > 3)
            {
                filteredText = filteredText.Substring(0, 3);
            }

            if (filteredText.StartsWith("0") && filteredText.Length > 1)
            {
                filteredText = filteredText.Substring(1);
            }

            if (filteredText == "0")
            {
                filteredText = "";
            }

            if (entry.Text != filteredText)
            {
                entry.Text = filteredText;
            }
        }
        private void OnWeightEntryTextChanged(object sender, TextChangedEventArgs e)
        {
            Entry entry = (Entry)sender;

            string filteredText = e.NewTextValue;

            if (filteredText.StartsWith(".") || filteredText.StartsWith(","))
            {
                filteredText = "0" + filteredText.Substring(1);
            }

            filteredText = new string(filteredText.Where(c => char.IsDigit(c) || c == '.' || c == ',').ToArray());

            if (filteredText.Count(c => c == '.' || c == ',') > 1)
            {
                filteredText = filteredText.Remove(filteredText.LastIndexOf('.') == filteredText.Length - 1 ? filteredText.LastIndexOf('.') : filteredText.LastIndexOf(','));
            }

            if (filteredText.Contains(".") || filteredText.Contains(","))
            {
                int decimalIndex = filteredText.IndexOf('.') > 0 ? filteredText.IndexOf('.') : filteredText.IndexOf(',');

                if (decimalIndex + 2 < filteredText.Length)
                {
                    filteredText = filteredText.Substring(0, decimalIndex + 2);
                }
            }

            if (filteredText.Contains(".") || filteredText.Contains(","))
            {
                var parts = filteredText.Split(new char[] { '.', ',' });

                if (parts[0].Length > 3)
                {
                    filteredText = parts[0].Substring(0, 3) + "." + parts[1];
                }
            }

            else if (filteredText.Length > 3)
            {
                filteredText = filteredText.Substring(0, 3);
            }

            if (filteredText.Length > 5)
            {
                filteredText = filteredText.Substring(0, 5);
            }

            if (filteredText == "0" || filteredText == "0." || filteredText == "0," || filteredText == "0.0" || filteredText == "0,0")
            {
                filteredText = "";
            }

            if (entry.Text != filteredText)
            {
                entry.Text = filteredText;
            }
        }
        private void OnStepGoalTextChanged(object sender, TextChangedEventArgs e)
        {
            Entry entry = (Entry)sender;

            string filteredText = new string(e.NewTextValue.Where(char.IsDigit).ToArray());

            if (filteredText.Length > 1 && filteredText.StartsWith("0"))
            {
                filteredText = filteredText.Substring(1);
            }

            if (filteredText.Length > 5)
            {
                filteredText = filteredText.Substring(0, 5);
            }

            if (filteredText == "0")
            {
                filteredText = "";
            }

            if (entry.Text != filteredText)
            {
                entry.Text = filteredText;
            }
        }
    }
}