const functions = require('firebase-functions');
const { initializeApp } = require('firebase/app');
const { getDatabase, ref, set } = require('firebase/database');
const admin = require('firebase-admin');

const firebaseConfig = {
  authDomain: "besafeplus-1d01d.firebaseapp.com",
  databaseURL: "https://besafeplus-1d01d-default-rtdb.europe-west1.firebasedatabase.app/", 
  projectId: "besafeplus-1d01d",
  storageBucket: "besafeplus-1d01d.appspot.com",
  messagingSenderId: "446458753408",
  appId: "1:446458753408:web:605ca3017077c254985165" 
};

initializeApp(firebaseConfig);

// Initialize the database
const db = getDatabase();

// Cloud Function to monitor service status
exports.checkServiceStatus = functions.database.ref('/service_status')
    .onUpdate(async (change, context) => {
        const beforeStatus = change.before.val();
        const afterStatus = change.after.val(); 

        // Normalize status value
        const normalizeStatus = (status) => {
            if (typeof status === 'boolean') {
                return status; 
            }
            return status === 'true';
        };

        const normalizedBeforeStatus = normalizeStatus(beforeStatus);
        const normalizedAfterStatus = normalizeStatus(afterStatus);

        // If service status is set to false, send a push notification
        if (normalizedAfterStatus === false && normalizedBeforeStatus !== false) {
            console.log('Service has been terminated. Sending push notification.');

            // Send a notification to all devices subscribed to 'general' topic
            try {
                await admin.messaging().sendToTopic('general', {
                    notification: {
                        title: 'Service Terminated',
                        body: 'The Activity Tracking Service has been terminated. Please restart it.'
                    }
                });
                console.log('Successfully sent message');
            } catch (error) {
                console.error('Error sending message:', error);
                throw new Error('Push notification failed');
            }
        }

        return null; 
    });
